#!/usr/bin/env bash

SCRIPT_PATH=$(cd $(dirname "$0")/ && pwd)

# OS to build on
DOCKER_OS=centos
DEFAULT_CENTOS_VERSION=centos7.9.2009
DEFAULT_UBUNTU_VERSION=18.04

AM2CM_SERVER_PROFILE=prod
AM2CM_DOCKER_IMAGE_VERSION=latest

AM2CM_VENV=venv

for i in "$@"; do
  case $i in
  -os=* | --docker-os=*)
    DOCKER_OS=${i#*=}
    shift
    ;;
  -iv=* | --image-version=*)
    AM2CM_DOCKER_IMAGE_VERSION=${i#*=}
    shift
    ;;
  -p=* | --profile=*)
    AM2CM_SERVER_PROFILE=${i#*=}
    shift
    ;;
  -d | --dev-mode)
    DEV_MODE=true
    shift
    ;;
  -h | --help)
    HELP=true
    shift
    ;;
  *)
    echo "Unknown option: $i"
    HELP=true
    ;;
  esac
done

__usage="
 Usage: $(basename "$0") [options]
 Options:
    -os, --docker-os: OS to build on
    -iv, --image-version: Set the version tag for the docker image
    -p, --profile: Set the active profile(s) for the am2cm-server
    -h, --help: Print this help
 "

if [ "${HELP}" = true ]; then
  echo "$__usage"
  exit 0
fi

if [ -z "${DOCKER_OS}" ]; then
  echo "ERROR: The OS for docker container is not specified!"
  exit 1;
fi

if [ -z "${AM2CM_SERVER_PROFILE}" ]; then
  echo "ERROR: The profile for the am2cm-server is not specified!"
  exit 1;
fi

case ${DOCKER_OS} in
  "centos") DOCKER_OS_VERSION=$DEFAULT_CENTOS_VERSION ;;
  "ubuntu") DOCKER_OS_VERSION=$DEFAULT_UBUNTU_VERSION ;;
   *)
     echo "ERROR: ${DOCKER_OS} is unsupported!!"
     exit 1
     ;;
esac

AM2CM_DOCKER_IMAGE=am2cm-"${DOCKER_OS}"


AM2CM_ROOT_DIR=/$(basename "$PWD")

SRC_AM2CM_SERVER=am2cm-server
SRC_AM2CM_ANSIBLE=am2cm-ansible
SRC_AM2CM_TOOL=am2cm-tool
SRC_AM2CM_UPGRADE=am2cm-upgrade
SRC_AM2CM_DIFF=am2cm-diff

if [ "${DEV_MODE}" = true ]; then
  SRC_AM2CM_SERVER=$SRC_AM2CM_SERVER/target/$SRC_AM2CM_SERVER
  SRC_AM2CM_ANSIBLE=$SRC_AM2CM_ANSIBLE/target/$SRC_AM2CM_ANSIBLE
  SRC_AM2CM_TOOL=$SRC_AM2CM_TOOL/target/$SRC_AM2CM_TOOL
  SRC_AM2CM_UPGRADE=am2cm-upgrade-tool/target/$SRC_AM2CM_UPGRADE
  SRC_AM2CM_DIFF=am2cm-diff/target/$SRC_AM2CM_DIFF
  AM2CM_SERVER_PROFILE=dev
fi

# It's important to add the docker profile to the end of the active profiles!!!
AM2CM_SERVER_PROFILE=${AM2CM_SERVER_PROFILE},docker

docker build -t "${AM2CM_DOCKER_IMAGE}":"${AM2CM_DOCKER_IMAGE_VERSION}" \
  --build-arg DOCKER_OS="${DOCKER_OS}" --build-arg DOCKER_OS_VERSION="${DOCKER_OS_VERSION}" \
  --build-arg AM2CM_ROOT_DIR="${AM2CM_ROOT_DIR}"  --build-arg AM2CM_SERVER_PROFILE="${AM2CM_SERVER_PROFILE}" \
  --build-arg SRC_AM2CM_SERVER=${SRC_AM2CM_SERVER} --build-arg SRC_AM2CM_ANSIBLE=${SRC_AM2CM_ANSIBLE} \
  --build-arg SRC_AM2CM_TOOL=${SRC_AM2CM_TOOL} --build-arg SRC_AM2CM_UPGRADE=${SRC_AM2CM_UPGRADE} \
  --build-arg SRC_AM2CM_DIFF=${SRC_AM2CM_DIFF} --build-arg AM2CM_VENV=${AM2CM_VENV} \
  -f docker/Dockerfile . \
  --target am2cm-final

docker tag "${AM2CM_DOCKER_IMAGE}":"${AM2CM_DOCKER_IMAGE_VERSION}" "${AM2CM_DOCKER_IMAGE}":latest
