#!/usr/bin/env bash

LOG_FILE=$(basename "$0" .sh).log

DEFAULT_DOCKER_OS=centos
DEFAULT_AM2CM_SERVER_PROFILE=prod
DEFAULT_PYTHON_EXECUTABLE=python3

for i in "$@"; do
  case $i in
  --local)
    SETUP_LOCAL=true
    ;;
  --docker)
    SETUP_DOCKER=true
    ;;
  -os=* | --docker-os=*)
    DOCKER_OS=${i#*=}
    shift
    ;;
  -p=* | --profile=*)
    AM2CM_SERVER_PROFILE=${i#*=}
    shift
    ;;
  -d | --dev-mode)
    DEV_MODE=true
    shift
    ;;
  -py=* | --python-executable=*)
    PYTHON_EXECUTABLE=${i#*=}
    shift
    ;;
  -h | --help)
    HELP=true
    shift
    ;;
  *)
    echo "Unknown option: $i"
    HELP=true
    ;;
  esac
done

__usage="
 Usage: $(basename "$0") [options]
 Options:
    --local: Install am2cm on the local machine
    --docker: Install am2cm in a docker container
    -os, --docker-os: the OS of the docker container
    -p, --profile: Set the active profile(s) for the am2cm-server
    -py, --python-executable: Path to python3 executable
    -h, --help: Print this help
 "

if [ "${HELP}" = true ]; then
  echo "$__usage"
  exit 0
fi

while [ -z "${SETUP_LOCAL}" ] && [ -z "${SETUP_DOCKER}" ]; do
  read -r -p "Select the type of installation ([l]ocal|[d]ocker):" INSTALL_TYPE
  case  $INSTALL_TYPE in
    l | local) SETUP_LOCAL=true;;
    d | docker) SETUP_DOCKER=true;;
    *) echo "ERROR: The '${INSTALL_TYPE}' is not a valid installation type! Please select one of the following options [l]ocal or [d]ocker"
      ;;
  esac
done

if [ "${SETUP_DOCKER}" = true ]; then
  if [ -z "${DOCKER_OS}" ]; then
    read -r -p "Enter the OS of the docker container [${DEFAULT_DOCKER_OS}]:" DOCKER_OS
    DOCKER_OS=${DOCKER_OS:-${DEFAULT_DOCKER_OS}}
  fi
fi

WORKING_DIR=${PWD}
if [ "$DEV_MODE" = true ]; then
  AM2CM_SERVER_PROFILE=dev
  if [ "${SETUP_LOCAL}" = true ]; then

    WORKING_DIR=${PWD}/am2cm-dist
    mkdir -p "${WORKING_DIR}"

    cp -r am2cm-server/target/am2cm-server "${WORKING_DIR}"
    cp -r am2cm-ansible/target/am2cm-ansible "${WORKING_DIR}"
    cp -r am2cm-tool/target/am2cm-tool "${WORKING_DIR}"

  else
    DEV_OPTS=--dev-mode
  fi
fi

if [ -z "${AM2CM_SERVER_PROFILE}" ]; then
  read -r -p "Enter the active profile for the am2cm-server [${DEFAULT_AM2CM_SERVER_PROFILE}]:" AM2CM_SERVER_PROFILE
  AM2CM_SERVER_PROFILE=${AM2CM_SERVER_PROFILE:-$DEFAULT_AM2CM_SERVER_PROFILE}
fi

if [ "${SETUP_DOCKER}" = true ]; then
  echo "Building docker image..." | tee -a "$LOG_FILE"
  ./build-docker.sh $DEV_OPTS -os="${DOCKER_OS}" --profile="${AM2CM_SERVER_PROFILE}" | tee -a "$LOG_FILE"
fi

if [ "${SETUP_LOCAL}" = true ]; then

  if [ -z "${PYTHON_EXECUTABLE}" ]; then
    read -r -p "Enter the path for python3 executable [${DEFAULT_PYTHON_EXECUTABLE}]:" PYTHON_EXECUTABLE
    PYTHON_EXECUTABLE=${PYTHON_EXECUTABLE:-${DEFAULT_PYTHON_EXECUTABLE}}
  fi

  VENV_DIR="${WORKING_DIR}"/venv
  echo "Creating virtualenv in ${VENV_DIR} ..." | tee -a "$LOG_FILE"
  "${PYTHON_EXECUTABLE}" -m venv "${VENV_DIR}" | tee -a "$LOG_FILE"

  echo "Installing requirements for am2cm-ansible in ${VENV_DIR} ..." | tee -a "$LOG_FILE"
  "${VENV_DIR}"/bin/pip install -r "${WORKING_DIR}"/am2cm-ansible/requirements.txt  | tee -a "$LOG_FILE"

  echo 'Configure am2cm-server ...' | tee -a "$LOG_FILE"
  cat >"${WORKING_DIR}"/am2cm-server/.env <<EOF
AM2CM_ROOT="${WORKING_DIR:-$PWD}"
ACTIVE_PROFILE=${AM2CM_SERVER_PROFILE}
EOF
fi

echo "Setup completed!" | tee -a "$LOG_FILE"